from .core import do_ini, backup_local, match_opt, match_active_opt, get_ini_lines

# if __name__ == '__main__':
#     do_ini("/etc/hosts", section="test", option="test", value="test", state="present")
#     do_ini("/etc/anotherconf", section="test", option="test", value="test", state="present")
#     do_ini("/etc/anotherconf", section="drinks", option="db", value="hu", state="present", backup=True)
#     do_ini("/etc/anotherconf", section="drinks", option="name", value="hu", state="present", backup=True, _diff=True)
#     do_ini("/etc/anotherconf", section="drinks", option="db3", value="", state="present", backup=True,
#            allow_no_value=True)
#     do_ini("/etc/anotherconf", section="drinks", option="db2", value="hu2", state="present", backup=True,
#            no_extra_spaces=True)
#     changed, backup_file, diff, msg = do_ini("/etc/anotherconf2", section="drinks", option="db", mode="get")
#     print(changed, backup_file, diff, msg['options'])



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import re
import shutil
import tempfile
import time
from .logs import logger
from .errors import FileCopyError, FileNotFoundError

log = logger(__name__)


def match_opt(option, line):
    """
    Match an option line.
    """
    option = re.escape(option)
    return re.match(' *%s( |\t)*=' % option, line) \
        or re.match('# *%s( |\t)*=' % option, line) \
        or re.match('; *%s( |\t)*=' % option, line)


# ==============================================================
# match_active_opt
def match_active_opt(option, line):
    """
    Match an active option line.
    """
    option = re.escape(option)
    return re.match(' *%s( |\t)*=' % option, line)


def backup_local(fn):
    """
    make a date-marked backup of the specified file, return True or False on success or failure
    :param fn:  file name
    :return:
    """
    # backups named basename-YYYY-MM-DD@HH:MM:SS~
    ext = time.strftime("%Y-%m-%d@%H:%M:%S~", time.localtime(time.time()))
    backupdest = '%s.%s' % (fn, ext)

    try:
        shutil.copy2(fn, backupdest)
    except (shutil.Error, IOError) as e:
        FileCopyError(msg='Could not make backup of %s to %s: %s' % (fn, backupdest, e))
    return backupdest


def get_ini_lines(filename):
    """
    读取INI文件内容，并返回每一行组成的列表。
    如果文件不存在或为空，则返回空列表。
    """
    if not os.path.exists(filename):
        return []

    with open(filename, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    # 确保每行都以换行符结尾
    for i in range(len(lines)):
        if not lines[i].endswith('\n'):
            lines[i] += '\n'

    return lines


def do_ini(filename, section=None, option=None, value=None,
           state='present', backup=False, no_extra_spaces=False,
           create=False, check_mode=False, _diff=False,
           allow_no_value=False, mode='set'):
    """
    :param filename:
    :param section:
    :param option:
    :param value:
    :param state:
    :param backup: 控制是否备份文件
    :param no_extra_spaces: 控制是否去除多余的空格
    :param create: 控制是否创建文件
    :param check_mode: 控制是否为检查模式
    :param _diff: 控制是否显示差异
    :param allow_no_value: 控制是否允许没有值
    :param mode: 控制操作模式，可选值为 'set' 或 'get'
    :return: changed, backup_file, diff, msg
    """
    if mode == 'get':
        ini_lines = get_ini_lines(filename)
        result = {
            'section': section,
            'option': option,
            'value': None,
            'options': {},
            'exists': False
        }
        in_section = False
        found_section = False

        for line in ini_lines:
            line = line.strip()
            if line.startswith('['):
                current_section = line[1:-1]
                if section and current_section == section:
                    in_section = True
                    found_section = True
                else:
                    in_section = False
            elif in_section and '=' in line:
                parts = line.split('=', 1)
                key = parts[0].strip()
                val = parts[1].strip() if len(parts) > 1 else None
                if key:
                    result['options'][key] = val
                    result['exists'] = True

        if not found_section:
            msg = "get not found in section '%s'" % section
        elif result['exists']:
            msg = "get option '%s' in section '%s'" % (option, section)
        else:
            msg = "option '%s' not found in section '%s'" % (option, section)
        return result['exists'], None, {}, {**result, 'msg': msg}

    if mode == 'set':
        diff = dict(
            before='',
            after='',
            before_header='%s (content)' % filename,
            after_header='%s (content)' % filename,
        )

        if not os.path.exists(filename):
            if not create:
                FileNotFoundError(msg='Destination %s does not exist !' % filename)
            destpath = os.path.dirname(filename)
            if not os.path.exists(destpath) and not check_mode:
                os.makedirs(destpath)
            ini_lines = []
        else:
            ini_file = open(filename, 'r')
            try:
                ini_lines = ini_file.readlines()
            finally:
                ini_file.close()

        if _diff:
            diff['before'] = ''.join(ini_lines)

        changed = False

        # ini file could be empty
        if not ini_lines:
            ini_lines.append('\n')

        # last line of file may not contain a trailing newline
        if ini_lines[-1] == "" or ini_lines[-1][-1] != '\n':
            ini_lines[-1] += '\n'
            changed = True

        # append fake section lines to simplify the logic
        # At top:
        # Fake random section to do not match any other in the file
        # Using commit hash as fake section name
        fake_section_name = "ad01e11446efb704fcdbdb21f2c43757423d91c5"

        # Insert it at the beginning
        ini_lines.insert(0, '[%s]' % fake_section_name)

        # At botton:
        ini_lines.append('[')

        # If no section is defined, fake section is used
        if not section:
            section = fake_section_name

        within_section = not section
        section_start = 0
        msg = 'OK'
        if no_extra_spaces:
            assignment_format = '%s=%s\n'
        else:
            assignment_format = '%s = %s\n'

        for index, line in enumerate(ini_lines):
            if line.startswith('[%s]' % section):
                within_section = True
                section_start = index
            elif line.startswith('['):
                if within_section:
                    if state == 'present':
                        # insert missing option line at the end of the section
                        for i in range(index, 0, -1):
                            # search backwards for previous non-blank or non-comment line
                            if not re.match(r'^[ \t]*([#;].*)?$', ini_lines[i - 1]):
                                if not value and allow_no_value:
                                    ini_lines.insert(i, '%s\n' % option)
                                else:
                                    ini_lines.insert(i, assignment_format % (option, value))
                                msg = 'option added'
                                changed = True
                                break
                    elif state == 'absent' and not option:
                        # remove the entire section
                        del ini_lines[section_start:index]
                        msg = 'section removed'
                        changed = True
                    break
            else:
                if within_section and option:
                    if state == 'present':
                        # change the existing option line
                        # 会遍历 ini_lines 并根据状态进行操作（添加、修改、删除）
                        if match_opt(option, line):
                            if not value and allow_no_value:
                                newline = '%s\n' % option
                            else:
                                newline = assignment_format % (option, value)
                            option_changed = ini_lines[index] != newline
                            changed = changed or option_changed
                            if option_changed:
                                msg = 'option changed'
                            ini_lines[index] = newline
                            if option_changed:
                                # remove all possible option occurrences from the rest of the section
                                index = index + 1
                                while index < len(ini_lines):
                                    line = ini_lines[index]
                                    if line.startswith('['):
                                        break
                                    if match_active_opt(option, line):
                                        del ini_lines[index]
                                    else:
                                        index = index + 1
                            break
                    elif state == 'absent':
                        # delete the existing line
                        if match_active_opt(option, line):
                            del ini_lines[index]
                            changed = True
                            msg = 'option changed'
                            break

        # remove the fake section line
        del ini_lines[0]
        del ini_lines[-1:]

        if not within_section and option and state == 'present':
            ini_lines.append('[%s]\n' % section)
            if not value and allow_no_value:
                ini_lines.append('%s\n' % option)
            else:
                ini_lines.append(assignment_format % (option, value))
            changed = True
            msg = 'section and option added'

        if _diff:
            diff['after'] = ''.join(ini_lines)

        log.debug("INI lines before processing: %s", ''.join(ini_lines))
        backup_file = None
        # 修改后的数据写入,只有当 changed == True 且 check_mode == False 时才会写入磁盘
        if changed and not check_mode:
            if backup:
                backup_file = backup_local(filename)
            try:
                tmpdir = tempfile.mkdtemp()
                tmpfd, tmpfile = tempfile.mkstemp(dir=tmpdir)
                f = os.fdopen(tmpfd, 'w')
                f.writelines(ini_lines)
                f.close()
            except Exception as e:
                raise e
            try:
                shutil.move(tmpfile, filename)
            except Exception as e:
                raise e
        return changed, backup_file, diff, msg

# ==============================================================
# main



#!/usr/bin/env python3
# -*- coding:utf8 -*-
class DIYError(Exception):
    ''' The base DIY exception from which all others should subclass '''

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class FileCopyError(DIYError):
    ''' Exception raised when a file copy fails '''
    pass


class FileNotFoundError(DIYError):
    ''' Exception raised when a file read fails '''
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev
import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'ini_crud'
DESCRIPTION = 'a simple tool for ini file crud'
URL = 'https://github.com/hujianli94/ini_crud.ini'
EMAIL = '1879324764@qq.com'
AUTHOR = 'hujianli'
REQUIRES_PYTHON = '>=3.8.0'
VERSION = '1.0.0'

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['django'],
}

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],
    install_requires=[],
    extras_require=EXTRAS,
    include_package_data=True,  # 包含非 Python 文件
    # package_data={
    #     'simple_ansible_ui': ['templates/*', 'public/*', 'logs/*'],
    # },
    entry_points={
        'console_scripts': ['ini_crud=ini_crud.cli:main'],
    },
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ]
)



#!/usr/bin/env python
# -*- coding:utf8 -*-
import os
from .utils import BaseModule
from .core import do_ini


# from logs import logger
#
# log = logger(__name__)


def main():
    module = BaseModule(
        argument_spec=dict(
            dest=dict(required=True, help="目标INI文件路径"),
            section=dict(required=True, help="要操作的section名称"),
            option=dict(required=False, help="要操作的option名称"),
            value=dict(required=False, help="设置的值"),
            backup=dict(default=False, type='bool', help="是否开启备份"),
            state=dict(default='present', choices=['present', 'absent'], help="状态：present/absent"),
            no_extra_spaces=dict(required=False, default=False, type='bool', help="是否移除空格"),
            create=dict(default=True, type='bool', help="如果文件不存在是否创建"),
        ),
        other_arg_spec=dict(
            check_mode=dict(default=False, type='bool', help="运行在check模式"),
            diff=dict(default=False, type='bool', help="显示配置差异"),
            allow_no_value=dict(default=True, type='bool', help="允许value为空"),
            mode=dict(default='set', type='str', choices=['set', 'get'], help="操作模式：set/get"),
        )
    )
    dest = os.path.expanduser(module.params.get('dest'))
    section = module.params.get("section")
    option = module.params.get("option", None)
    value = module.params.get("value", None)
    state = module.params.get("state")
    backup = module.params.get("backup")
    no_extra_spaces = module.params['no_extra_spaces']
    create = module.params['create']
    check_mode = module.params['check_mode']
    has_diff = module.params['diff']
    has_allow_no_value = module.params['allow_no_value']
    mode = module.params['mode']

    # log.debug(
    #     "dest:{} \nsection:{} \noption:{} \nvalue:{} \nstate:{} \nbackup:{} \nno_extra_spaces:{} \ncreate:{} \ncheck_mode:{} \nhas_diff:{} \nhas_allow_no_value:{} \nmode:{}".format(
    #         dest, section, option, value, state, backup, no_extra_spaces, create, check_mode, has_diff,
    #         has_allow_no_value, mode)
    # )
    (changed, backup_file, diff, msg) = do_ini(dest, section, option, value, state, backup, no_extra_spaces,
                                               create, check_mode, has_diff, has_allow_no_value, mode)

    results = {'changed': changed, 'stdout': msg, 'dest': dest, 'diff': diff, 'mode': mode}
    if backup_file is not None:
        results['backup_file'] = backup_file

    module.exit_json(**results)



# -*- coding: utf-8 -*-
import sys
import json
from argparse import ArgumentParser, SUPPRESS
from typing import Dict, List, Any, Optional

"""
# 命令行示例
python cli.py name=api state=present port=80 enabled=yes tags=api,web config='{"env":"prod"}'

# json格式
python cli.py '{"name": "api", "state": "present", "port": 80, "enabled": true, "tags": ["api", "web"], "config": {"env": "prod"}}'
"""


class BaseModule:
    def __init__(
            self,
            argument_spec: Dict[str, Dict[str, Any]],
            required_one_of: List[List[str]] = None,
            mutually_exclusive: List[List[str]] = None,
            required_together: List[List[str]] = None,
            required_if: List[List[str]] = None,
            supports_check_mode: bool = False,
            add_file_common_args: bool = False,
            other_arg_spec: Dict[str, Any] = None
    ):
        self.argument_spec = argument_spec
        self.supports_check_mode = supports_check_mode  # 支持check模式
        self.other_arg_spec = other_arg_spec or {}

        # 解析参数
        self.params = self._parse_arguments()
        self._validate_arguments()

    def _parse_arguments(self) -> Dict[str, Any]:
        """解析命令行参数（支持标准Ansible格式）"""
        parser = ArgumentParser(add_help=True)  # 启用默认帮助支持
        parser.add_argument('args', nargs='*', help=SUPPRESS)
        # 动态添加参数和帮助信息
        for key, spec in {**self.argument_spec, **self.other_arg_spec}.items():
            help_text = spec.get('help', '')  # 获取帮助信息
            action = 'store_true' if spec.get('type') == 'bool' else None
            parser.add_argument('--{}'.format(key), dest=key, action=action, help=help_text)
        args, _ = parser.parse_known_args()

        # 如果没有传入任何参数，则打印帮助信息
        if len(sys.argv) <= 1 or (len(args.args) == 0 and not self._is_json(sys.argv[-1])):
            parser.print_help()
            sys.exit(1)
        params: Dict[str, Any] = {}
        # 优先解析JSON格式参数（最后一个参数为JSON）
        if len(args.args) >= 1 and self._is_json(args.args[-1]):
            params = json.loads(args.args[-1])
            args.args = args.args[:-1]  # 移除已解析的JSON参数

        # 解析key=value参数
        for arg in args.args:
            if '=' in arg:
                key, value = arg.split('=', 1)
                # 处理带引号的参数值
                if value.startswith(("'", '"')) and value.endswith(("'", '"')):
                    value = value[1:-1]  # 移除引号
                params[key] = self._convert_type(key, value)

        # 合并 argument_spec 和 other_arg_spec 的默认值和类型转换
        for key, spec in {**self.argument_spec, **self.other_arg_spec}.items():
            if key not in params and 'default' in spec:
                params[key] = spec['default']
            elif key in params:
                # 如果已存在，再做一次类型转换（确保布尔值等正确）
                params[key] = self._convert_type(key, str(params[key]))

        # 处理check_mode参数
        if self.supports_check_mode and 'check_mode' in params:
            self.check_mode = params['check_mode']
        else:
            self.check_mode = False  # 默认关闭

        return params

    @staticmethod
    def _is_json(s: str) -> bool:
        """判断是否为JSON字符串"""
        try:
            json.loads(s)
            return True
        except json.JSONDecodeError:
            return False

    def _convert_type(self, key: str, value: str) -> Any:
        """根据argument_spec转换参数类型"""
        spec = self.argument_spec.get(key, {})
        type_ = spec.get('type', 'str')

        if type_ == 'int':
            return int(value)
        elif type_ == 'bool':
            if value.lower() in ('false', 'no', 'off', '0', 'f', 'n'):
                return False
            if value.lower() in ('true', 'yes', 'on', '1', 't', 'y'):
                return True
        elif type_ == 'list':
            return value.split(',') if value else []
        elif type_ == 'dict':
            return json.loads(value) if self._is_json(value) else {}
        elif type_ == 'path':
            return value  # 路径类型可自定义处理
        return value  # 字符串直接返回

    def _validate_arguments(self):
        """校验必选参数和类型"""
        errors = []
        # 校验required参数
        for key, spec in self.argument_spec.items():
            if spec.get('required', False) and key not in self.params:
                errors.append("Required parameter missing: {}".format(key))
        # 校验choices参数
        for key, spec in self.argument_spec.items():
            choices = spec.get('choices')
            if choices and self.params.get(key) not in choices:
                errors.append("Invalid value for {}: {}, must be in {}".format(key, self.params.get(key), choices))
        if errors:
            self.fail_json("\n".join(errors))

    def exit_json(self, changed: bool = False, msg: str = "Success", **kwargs) -> None:
        """返回标准JSON结果"""
        result = {"changed": changed, "msg": msg, **self.params, **kwargs}
        if self.supports_check_mode:
            result["check_mode"] = self.check_mode
        print(json.dumps(result, ensure_ascii=False, indent=2))
        sys.exit(0)

    def fail_json(self, msg: str, **kwargs) -> None:
        """返回错误JSON结果"""
        result = {"failed": True, "msg": msg, **self.params, **kwargs}
        print(json.dumps(result, ensure_ascii=False, indent=2))
        sys.exit(1)

# def main():
#     # 定义标准argument_spec格式
#     module_args = dict(
#         name=dict(required=True, type='str'),
#         state=dict(
#             required=False,
#             type='str',
#             choices=['present', 'absent'],
#             default='present'
#         ),
#         port=dict(type='int', default=8080),
#         enabled=dict(type='bool', default=False),
#         tags=dict(type='list'),
#         config=dict(type='dict'),
#     )
#
#     # 初始化模块（支持check_mode等特性）
#     module = AnsibleModule(
#         argument_spec=module_args,
#         supports_check_mode=True
#     )
#
#     # 业务逻辑示例
#     try:
#         # 处理check_mode
#         if module.check_mode:
#             module.exit_json(changed=False, msg="Check mode: 无实际操作")
#
#         # 模拟业务操作
#         result = {
#             "changed": True,
#             "msg": "资源 {} 状态为 {}.".format(module.params["name"], module.params['state']),
#             "port": module.params["port"],
#             "enabled": module.params["enabled"]
#         }
#         # 添加可选参数
#         if module.params.get('tags'):
#             result['tags'] = module.params['tags']
#         if module.params.get('config'):
#             result['config'] = module.params['config']
#
#         module.exit_json(**result)
#
#     except Exception as e:
#         module.fail_json(msg=str(e))
#
#
# if __name__ == "__main__":
#     main()
